/*-------------------------------------------------
ファイル名:serial_test.c
説明：
SmartLCDCとシリアル通信の確認を行うソフト
SmartLCDCへ画面に触れている間にコマンドを返送する「t2」コマンドを送信する。
ラズベリーパイが受信したコマンドをコンソールへ表示する。

 コンパイル方法:
gcc -o serial_test1 serial_test1.c -I/usr/local/include -L/usr/local/lib -lwiringPi
コンパイル時のオプションの説明。
        -I共有ヘッダファイルの場所) -L(共有ライブラリの場所) -l(共有ライブラリ $
-------------------------------------------------*/
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<unistd.h>

//wiringPiのインクルード
#include<wiringPi.h>
#include<wiringSerial.h>

#define CR 0x0d

int fd;
unsigned short inCount,outCount,usBufCount;
char rBuffer[500];
char cBuffer[500];




int fnSciInit(void){
        //sciの初期化
        fd = serialOpen("/dev/ttyUSB0",115200);
        if(fd < 0){
                return -1;
        }
        inCount = 0;
        outCount = 0;

	return 0;
}

/*------------------------------------------
関数名　fnInit
説明
　各機能の初期化を行います。
------------------------------------------*/
int fnInit(void){

        //Sciの初期化
        if(fnSciInit() < 0){
                printf("ttyUSB0 not open in fnInit\r\n");
        }
}
/*----------------------------------
関数名 fnputus
説明
        ラズベリーパイからSmartLCDCへ送信する関数。
----------------------------------*/
void fnputus(char *cp){
 	serialPrintf(fd,cp);
	serialPrintf(fd,"\r");
}
/*----------------------------------
関数名 fnSerialRead
説明
	SmartLCDCからのコマンドを受信した場合、文字列を読み取りバッファーへ格納する関数。
----------------------------------*/
void fnSerialRead(void){
        while(serialDataAvail(fd)){
                rBuffer[inCount] = serialGetchar(fd);
                fflush(stdout);
//              putchar("%c",rBuffer[inCount]);
//              printf("\r\n");
                inCount++;
                if(inCount >= 500){
                        inCount = 0;
                }
        }
}
/*----------------------------------
関数名 fnSerial_Decision
説明
        SmartLCDCからのコマンドを受信したコマンドを解析する関数。
----------------------------------*/
void fnSerial_Decision(void){
        char *ptr;

        fnSerialRead();		//SmartLCDCからの受信をバッファに格納する関数。

        //コマンドの文字読み取り部 ループバッファ処理
        for(;;){
                if(inCount == outCount) break;
                if(inCount != outCount){
                        cBuffer[usBufCount] = rBuffer[outCount];
                        usBufCount++;
                        if(usBufCount >= 499) cBuffer[0] = 0x0d;
                        if(++outCount > 499) outCount = 0;
                }
                if((usBufCount >= 1) && (cBuffer[usBufCount-1] == CR)) break;
        }
        //コマンド解析部
        if((usBufCount >= 1) && (cBuffer[usBufCount-1] == CR)){
                usBufCount = 0;

                ptr = cBuffer;
                switch(*ptr){	//受信したコマンド1文字目を調べる。
                        case 't':
				//fnt(ptr);
				printf(ptr);	//受信したコマンドをそのままコンソールに表示。
				char cBuf[50] = "F0100A0910000FFFF";
				fnputus(strcat(cBuf,ptr));	//受信したコマンドをSmartLCDCへ表示する
				break;
                        default:
                                break;
                }
        }
}

void main(void){

	fnInit();	//シリアル通信部の初期化

//	serialPrintf(fd,"t2\r");	//「t2」コマンドを送信する。SmartLCDCのタッチパネルに触れている間、SmartLCDCがタッチパネルデータを返送するようになるコマンド

	fnputus("t2");

//	fnputus("P00000000000\r");		//画像00000.bmpを左上の座標(000,000)に表示させるコマンド

	while(1){

		fnSerial_Decision();
		usleep(10000);		//10mS待つ

	}
}
